#include <IRremote.h>

// Motor A pins
#define MOTOR_A_IN1 10
#define MOTOR_A_IN2 9

// Motor B pins
#define MOTOR_B_IN1 6
#define MOTOR_B_IN2 5

// IR receiver pin
#define IR_RECEIVER_PIN 2

// IR command codes
#define UNIQUE_ID 69
#define FORWARD_CODE 24
#define STOP_CODE 28
#define REVERSE_CODE 82
#define RIGHT_CODE 90
#define LEFT_CODE 8

// Motor speed settings (0-255)
#define STANDARD_SPEED 160
#define HI_SPEED 250
// Timing variables
unsigned long lastIdTime = 0;
const unsigned long ID_TIMEOUT = 1000; // 1 seconds in milliseconds
boolean idReceived = false;

// Turn duration
const unsigned long TURN_DURATION = 1000; // 1 second in milliseconds
unsigned long turnEndTime = 0;
boolean isTurning = false;

void setup() {
  // Set motor control pins as outputs
  pinMode(MOTOR_A_IN1, OUTPUT);
  pinMode(MOTOR_A_IN2, OUTPUT);
  pinMode(MOTOR_B_IN1, OUTPUT);
  pinMode(MOTOR_B_IN2, OUTPUT);

  // Initialize motors to stopped state
  stopMotors();

  // Start serial communication for debugging
  Serial.begin(9600);
  
  // Start the IR receiver
  IrReceiver.begin(IR_RECEIVER_PIN, ENABLE_LED_FEEDBACK);
  
  Serial.println("Robot initialized, waiting for commands...");
}

void loop() {
  // Check if currently in a turning state
  if (isTurning && millis() >= turnEndTime) {
    stopMotors();
    isTurning = false;
    Serial.println("Turn completed, stopped");
  }

  // Check for IR signal
  if (IrReceiver.decode()) {
    uint16_t command = IrReceiver.decodedIRData.command;
    
    Serial.print("IR Code received: ");
    Serial.println(command);
    
    // Check if unique ID received
    if (command == UNIQUE_ID) {
      idReceived = true;
      lastIdTime = millis();
      Serial.println("Unique ID received");
    }
    // Process command if ID was recently received
    else if (idReceived && (millis() - lastIdTime < ID_TIMEOUT)) {
      processCommand(command);
      idReceived = false; // Reset ID flag after processing a command
    }
    
    IrReceiver.resume(); // Receive the next value
  }
  
  // Check if ID has timed out
  if (idReceived && (millis() - lastIdTime >= ID_TIMEOUT)) {
    idReceived = false;
    Serial.println("Unique ID timeout");
  }
}

void processCommand(uint16_t command) {
  Serial.print("Processing command: ");
  Serial.println(command);
  
  switch (command) {
    case FORWARD_CODE:
      moveForward();
      break;
    case STOP_CODE:
      stopMotors();
      break;
    case REVERSE_CODE:
      moveBackward();
      break;
    case RIGHT_CODE:
      turnRight();
      break;
    case LEFT_CODE:
      turnLeft();
      break;
    default:
      Serial.println("Unknown command");
      break;
  }
}

void moveForward() {
  Serial.println("Moving forward");
  // Motor A forward
  analogWrite(MOTOR_A_IN1, STANDARD_SPEED);
  analogWrite(MOTOR_A_IN2, 0);
  // Motor B forward
  analogWrite(MOTOR_B_IN1, STANDARD_SPEED);
  analogWrite(MOTOR_B_IN2, 0);
}

void moveBackward() {
  Serial.println("Moving backward");
  // Motor A backward
  analogWrite(MOTOR_A_IN1, 0);
  analogWrite(MOTOR_A_IN2, STANDARD_SPEED);
  // Motor B backward
  analogWrite(MOTOR_B_IN1, 0);
  analogWrite(MOTOR_B_IN2, STANDARD_SPEED);
}

void turnRight() {
  Serial.println("Turning right");
  // Motor A forward
  analogWrite(MOTOR_A_IN1, STANDARD_SPEED);
  analogWrite(MOTOR_A_IN2, 0);
  // Motor B backward
  analogWrite(MOTOR_B_IN1, 0);
  analogWrite(MOTOR_B_IN2, STANDARD_SPEED);
  
  // Set timer for turn duration
  turnEndTime = millis() + TURN_DURATION;
  isTurning = true;
}

void turnLeft() {
  Serial.println("Turning left");
  // Motor A backward
  analogWrite(MOTOR_A_IN1, 0);
  analogWrite(MOTOR_A_IN2, STANDARD_SPEED);
  // Motor B forward
  analogWrite(MOTOR_B_IN1, STANDARD_SPEED);
  analogWrite(MOTOR_B_IN2, 0);
  
  // Set timer for turn duration
  turnEndTime = millis() + TURN_DURATION;
  isTurning = true;
}

void stopMotors() {
  Serial.println("Stopping motors");
  // Motor A stop
  analogWrite(MOTOR_A_IN1, 0);
  analogWrite(MOTOR_A_IN2, 0);
  // Motor B stop
  analogWrite(MOTOR_B_IN1, 0);
  analogWrite(MOTOR_B_IN2, 0);
}